function [m,e] = mymodelgmm(b,infoz,stat,y,x,z)
% PURPOSE:  Provide moment conditions and error term for 
%             linear GMM estimation
%-------------------------------------------------------------------------
% USAGE:  [m,e] = lingmmm(b,infoz,stat,y,x,z,w)
%  b      model parameters
%  infoz   MINZ infoz structure
%  stat   MINZ status structure
%  y,x,z  Data: dependent, independent, and instruments
%  w      GMM weighting matrix
%-------------------------------------------------------------------------
% RETURNS:
%  m      vector of moment conditions
%  e      Model errors  (Nobs x Neq)
%-------------------------------------------------------------------------
% VERSION: 1.1.2

% written by:
% Mike Cliff,  Purdue Finance  mcliff@mgmt.purdue.edu
% Created: 12/10/98
% Modified 9/26/00 (1.1.1 Does system of Eqs)
%          11//13/00 (1.1.2 No W as input argument)
  
b1 = b(1);
b2 = b(2);
b3 = b(3);
b4 = b(4);
b5 = b(5);
b6 = b(6);

x1=x(:,1);
x2=x(:,2);
x3=x(:,3);

e1 = y(:,1) - b1*(x1.^b2 + x2.^b2);
e2 = y(:,2) - b3 - b4*x1 - b5*x2 - b6*x3;

e = [e1 e2];
% z1 = [ones(1000,1) x1 x2];
% z2 = [ones(1000,1) x1];

m1 = z'*e1/rows(e1);
m2 = z'*e2/rows(e2);

m = [m1; m2];
% m = vec(z'*e/rows(e));

